#pragma once

#include <RobotAPI/libraries/armem/core/MemoryID.h>

namespace armarx::armem::client::util
{

    class MemoryListener;

    class SubscriptionHandle
    {
        friend class MemoryListener;

    public:
        SubscriptionHandle();
        SubscriptionHandle(SubscriptionHandle&& other);

        /**
        * @brief Assignment operator.
        *
        * @note Intentional call by value, since this leverages the move constructor. See
        * https://stackoverflow.com/a/11540204 (section "Move assignment operators").
        */
        SubscriptionHandle& operator=(SubscriptionHandle other);

        friend void swap(SubscriptionHandle& first, SubscriptionHandle& second);

        void release();

    private:
        SubscriptionHandle(MemoryListener* memoryListener, const MemoryID& memoryID, long id);

    private:
        bool valid = false;
        MemoryListener* memoryListener = nullptr;
        MemoryID memoryID;
        long id = 0;
    };

    class ScopedSubscriptionHandle
    {
    public:
        ScopedSubscriptionHandle();
        ScopedSubscriptionHandle(SubscriptionHandle&& handle);

        /**
        * @brief Assignment operator.
        *
        * @note Intentional call by value, since this leverages the move constructor. See
        * https://stackoverflow.com/a/11540204 (section "Move assignment operators").
        */
        ScopedSubscriptionHandle& operator=(SubscriptionHandle handle);

        ~ScopedSubscriptionHandle();

    private:
        SubscriptionHandle handle;
    };

    void swap(SubscriptionHandle& first, SubscriptionHandle& second);

} // namespace armarx::armem::client::util
